'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.paginationToParams = paginationToParams;
exports.dataToConnection = dataToConnection;

var _base = require('./base64');

var PREFIX = 'dynamodbconnection:';

function paginationToParams(_ref) {
  var first = _ref.first;
  var after = _ref.after;

  var params = { Limit: first };
  if (after) params.ExclusiveStartKey = { id: cursorToId(after) };
  return params;
}

/**
 * Accepts the `data` (serialized response) from DynamoDB
 * and converts that output to a Connection object that
 * follows the (GraphQL) Relay Cursor Connections Specification:
 * https://facebook.github.io/relay/graphql/connections.htm.
 *
 * Inspired by the `graphql-relay-js` `connection/arrayconnection.js`
 * https://github.com/graphql/graphql-relay-js/blob/87d865e0623d6ee0c799dcbc\
 *   266e9e4c68bfc5d3/src/connection/arrayconnection.js
 */

function dataToConnection(_ref2) {
  var Items = _ref2.Items;
  var LastEvaluatedKey = _ref2.LastEvaluatedKey;

  var edges = Items.map(function (value) {
    return {
      cursor: idToCursor(value.id),
      node: value
    };
  });

  var firstEdge = edges[0];
  var lastEdge = edges[edges.length - 1];

  return {
    edges: edges,
    pageInfo: {
      startCursor: firstEdge ? firstEdge.cursor : null,
      endCursor: lastEdge ? lastEdge.cursor : null,
      hasPreviousPage: false,

      //TODO: fix bug: if count=2 and first=2 LastEvaluatedKey will exist
      //not sure how to resolve this edge case yet
      hasNextPage: !!LastEvaluatedKey
    }
  };
}

var idToCursor = function idToCursor(id) {
  return (0, _base.base64)(PREFIX + id);
};
var cursorToId = function cursorToId(cursor) {
  return (0, _base.unbase64)(cursor).substring(PREFIX.length);
};